context("dummy encoding")

#############
### SETUP ###
#############

expected_df_both <- data.frame(y = y,
                               x1b = c(0, 1, NA, 1, 0, 0),
                               x2d = c(0, 0, 0, 1, 1, 0))

expected_tbl_both <- tibble::as_tibble(expected_df_both)

expected_x1_df_fact <- data.frame(y, x2, x1b = c(0, 1, NA, 1, 0, 0))

expected_x1_df_char <- data.frame(y,
                                  x2,
                                  x1b = c(0, 1, NA, 1, 0, 0),
                                  stringsAsFactors = FALSE)

expected_x1_tbl_char <- tibble::tibble(y,
                                       x2,
                                       x1b = c(0, 1, NA, 1, 0, 0))

expected_x1_tbl_fact <- tibble::tibble(y,
                                       x2 = factor(x2),
                                       x1b = c(0, 1, NA, 1, 0, 0))

###################################
### MULTIPLE TRAINING ENCODINGS ###
###################################

test_that("catto_dummy: multiple data.frame training columns.", {

  both_encoded <- check_x1_x2(catto_dummy, "data.frame")
  for (m in both_encoded) expect_equal(m, expected_df_both)

})

test_that("catto_dummy: multiple tibble training columns.", {

  both_encoded <- check_x1_x2(catto_dummy, "tibble")
  # TODO: make sure this returns a tibble
  for (m in both_encoded) expect_equal(m, expected_df_both)

})

##########################
### ONE TRAIN ENCODING ###
##########################

test_that("catto_dummy: one data.frame training column.", {

  one_encoded <- check_x1(catto_dummy, "data.frame")
  num_tests <- length(one_encoded)

  for (i in seq(from = 1, to = num_tests / 2)) {
    expect_equal(one_encoded[[i]], expected_x1_df_fact)
  }

  for (i in seq(from = num_tests / 2 + 1, to = num_tests)) {
    expect_equal(one_encoded[[i]], expected_x1_df_char)
  }

})

test_that("catto_dummy: one tibble training column.", {

  one_encoded <- check_x1(catto_dummy, "tibble")
  num_tests <- length(one_encoded)

  for (i in seq(from = 1, to = num_tests / 2)) {
    expect_equal(one_encoded[[i]], expected_x1_df_fact)
  }

  for (i in seq(from = num_tests / 2 + 1, to = num_tests)) {
    expect_equal(one_encoded[[i]], expected_x1_df_char)
  }

})
#################
### TEST DATA ###
#################

#test_that("catto_dummy correctly encodes test data.", {

#  test_df <- data.frame(y = seq_len(3),
#                        x1 = c("e", NA, "b"),
#                        x2 = c("c", "d","c"))

#  expected_test <- data.frame(y = seq_len(3),
#                              x2 = c("c", "d", "c"),
#                              x1b = c(NA, NA, 1))

#  expect_equal(catto_dummy(df_fact, x1, test = test_df),
#               list(train = expected_x1, test = expected_test))

#})

###